#include <chrono>
#include <vector>
#include <iostream>
#include <algorithm>
#include <functional>

using namespace std::chrono;

class Tour 
{
public:
    enum Level
    {
        TWO = 2,
        THREE,
        FOUR,
        FIVE
    };

    Tour(const std::string& country = "",
         const uint32_t rest_days_amount = 0,
         const uint32_t people_amount = 0,
         const Level level = Level::TWO,
         const year_month_day& departure_date = 1111y / 11 / 11,
         const float price = 0) 
    {
        this->country = country;
        this->rest_days_amount = rest_days_amount;
        this->people_amount = people_amount;
        this->level = level;
        this->departure_date = departure_date;
        this->price = price;
    }


    Tour& setCountry(const std::string& country) 
    {
        this->country = country;
        return *this;
    }

    std::string getCountry() 
    {
        return this->country;
    }
    

    Tour& setRestDaysAmount(const uint32_t rest_days_amount) 
    {
        this->rest_days_amount = rest_days_amount;
        return *this;
    }

    uint32_t getRestDaysAmount()
    {
        return this->rest_days_amount;
    }


    Tour& setPeopleAmount(const uint32_t people_amount) 
    {
        this->people_amount = people_amount;
        return *this;
    }

    uint32_t getPeopleAmount()
    {
        return this->people_amount;
    }


    Tour& setLevel(const Level level) 
    {   
        this->level = level;
        return *this;
    }

    Level getLevel() 
    {
        return this->level;
    }


    Tour& setDepartureDate(const year_month_day& departure_date) 
    {
        this->departure_date = departure_date;
        return *this;
    }

    year_month_day getDepartureDate() 
    {
        return this->departure_date;
    }


    Tour& setPrice(const float price) 
    {
        this->price = price;
        return *this;
    }

    float getPrice() 
    {
        return this->price;
    }


    void showAll();

private:
    std::string country;
    uint32_t rest_days_amount;
    uint32_t people_amount;
    Level level;
    year_month_day departure_date;
    float price;
};

void Tour::showAll() 
{
    std::cout << "Country: " << this->country << ", "
              << "Rest days: " << this->rest_days_amount << ", "
              << "People: " << this->people_amount << ", "
              << "Level: " << this->level << ", "
              << "Departure date: " << this->departure_date << ", "
              << "Price: " << this->price << std::endl;
}

int main()
{ 
    std::vector<Tour> repository { Tour("Russia"), 
                                   Tour("Spain"),
                                   Tour("Ukraine").setPrice(15400).setLevel(Tour::Level::FIVE).setRestDaysAmount(7),
                                   Tour("Spain").setPeopleAmount(4),
                                   Tour("China").setDepartureDate(2022y / September / 12)
                                 };

    decltype(repository) a, b, c;

    std::copy_if(repository.begin(), repository.end(), std::back_inserter(a), [](Tour& t) { return t.getCountry() == "Spain"; });
    std::copy_if(repository.begin(), repository.end(), std::back_inserter(b), [](Tour& t) {
        return t.getPrice() <= 20000.f &&
               t.getLevel() >= Tour::Level::FOUR &&
               t.getRestDaysAmount() == 7;
    });
    std::copy_if(repository.begin(), repository.end(), std::back_inserter(c), [](Tour& t) {
        auto date_now = std::chrono::system_clock::now();
        auto days_diffenece = duration_cast<std::chrono::days>(sys_days(t.getDepartureDate()) - date_now);
        return days_diffenece.count() >= 15;
    });

    std::cout << "A)\n";
    std::for_each(a.begin(), a.end(), std::mem_fn(&Tour::showAll));

    std::cout << "\nB)\n";
    std::for_each(b.begin(), b.end(), std::mem_fn(&Tour::showAll));

    std::cout << "\nC)\n";
    std::for_each(c.begin(), c.end(), std::mem_fn(&Tour::showAll));
}
